<?php

namespace Teknisa\EST\Services;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Teknisa\Model\Estoque\Entities\Lanctoestoq;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\DateUtil;

class ProdProcessaService {

    protected $entityManager;
    protected $crudService;
    protected $environment;
    protected $connection;
    protected $newCode;

    public function __construct($entityManager, $crudService, $environment, $newCode) {
        $this->entityManager = $entityManager;
        $this->crudService = $crudService;
        $this->environment = $environment;
        $this->connection = $entityManager->getConnection();
        $this->newCode = $newCode;
    }

    public function doStockOperations(&$retirada) {
      try{
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();  //Desativa auto commit
        
        //Parte Referente a Entrada.
        //realiza o lançamento de estoque  referente a saída e entrada  ------ 4-Saida 2 -Entrada
        $nrctrlmovest = $retirada['NRLANCPROCESSA'];
        $nrlancamentsaida    = $this->saveTabelaLancamentoEstoque($retirada,$nrctrlmovest,4);
        $nrlancamentoentrada = $this->saveTabelaLancamentoEstoque($retirada,$nrctrlmovest,2);
        $retirada['NRLANCSAID'] = $nrlancamentsaida;
        $retirada['NRLANCENTR'] = $nrlancamentoentrada;
        
        
        $conditions =  array();
        $filterRow = new FilterCriteria('itlancprocessa', $conditions);
        $filterRow->setDataSourceName('itlancprocessa');
        $filterRow->addCondition('CDFILIAL', $retirada['CDFILIAL']);
        $filterRow->addCondition('NRLANCPROCESSA', $retirada['NRLANCPROCESSA']);
        $rowResponse = new DTO\Response;
        $this->crudService->find($rowResponse, $filterRow);
        $rowsEntrada = $rowResponse->getDataSets()[0]->getRows();
        
        //realiza o lançamento de item de saída e entrada  ------ 4-Saida 2 -Entrada
        $this->saveTabelaItemLancamentoEstoque($nrctrlmovest, $retirada, $rowsEntrada, 2, $nrlancamentoentrada);
        $this->saveTabelaItemLancamentoEstoque($nrctrlmovest, $retirada, [$retirada], 4, $nrlancamentsaida, $nrsequitemrequest = true);
        
        
        var_dump('afteritems');die();

        $connection->commit();
      }catch (\Exception $e) {
        var_dump('no exception');
        $this->entityManager->rollback(); //rollback em caso de erro
        var_dump($e);die;
        throw $e;
      }
      
      /*$dtProcessa = new \DateTime($fullDate[2].'-'.$fullDate[1].'-'.$fullDate[0]);
      $paramfilial = $this->entityManager->getRepository('\Geral\Entities\Paramfilial')
        ->findOneBy(
          array(
            'cdfilial' => $cdfilial
          )
        );
      $paramfilial->setDtprocessa($dtProcessa);
      $this->entityManager->persist($paramfilial);
      $this->entityManager->flush();*/
      
    }
    
    public function saveTabelaLancamentoEstoque($row,$nrctrlmovest,$identradaSaida){
      
      $nrorg = $this->environment->getNrOrgTrab() || 1;
      $operador = $this->environment->getCdOperador();
      $currentTime = $this->environment->getCurrentTime();
      
      $nrlancestq = $this->newCode->getCodeByTableName('LANCTOEST'.$row['CDFILIAL'], $nrorg ,1,10);
      
      $lanctoestoq = new Lanctoestoq();
      //campos auditoria
      $lanctoestoq->setNrorg($nrorg);
      $lanctoestoq->setDtinclusao($currentTime);
      $lanctoestoq->setCdoperinclusao($operador);
      $lanctoestoq->setNrorginclusao($nrorg);
      $lanctoestoq->setNrorgultatu($nrorg);
      $lanctoestoq->setDtultatu($currentTime);
      $lanctoestoq->setCdoperultatu($operador);
      $lanctoestoq->setIdativo("S");

      $lanctoestoq->setCdfilial($row['CDFILIAL']);
      $lanctoestoq->setNrlancestq($nrlancestq);
      $lanctoestoq->setIdtplancto($identradaSaida);
      $data = substr($row['DTLANCPROCESSA'], 0, 10);
      $lanctoestoq->setDtlancestq(DateUtil::getDataDeString($data));
      //Processamento 
      $lanctoestoq->setDslancestq('Processamento '.$nrctrlmovest.' do produto '.$row['NMPRODUTO']);
      $lanctoestoq->setCdfilimovi($row['CDFILIAL']);
      $lanctoestoq->setCdoperador($operador);

      $this->entityManager->persist($lanctoestoq);
      $this->entityManager->flush();
      
      return $nrlancestq;
    }
    
    public function saveTabelaItemLancamentoEstoque($nrctrlmovest, $rowEntrada, $produtos, $identradaSaida, $nrlancamentoestoque, $saveNRSEQITEMREQUEST = false){
        try {
            $nrorg       = $this->environment->getNrorgTrab() || 1;
            $operador    = $this->environment->getCdOperador();
            $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
            $arrayItens  = [];
            $nrtipocusto = $this->buscaTipoCustoFilial($nrorg,$rowEntrada['CDFILIAL']);
            $nrtipocusto = $nrtipocusto? $nrtipocusto[0]['CDTIPCUSTEST'] : $nrtipocusto;
            
            $countproduto = 0;
            
            foreach ($produtos as &$prod) {
                $cdalmoxarife = '';
                $cdlocalestoq = '';

                //item de saida
                if($identradaSaida ==4){
                    $cdalmoxarife = $rowEntrada['CDALMOXSAID']?? ' ';
                    $cdlocalestoq = $rowEntrada['CDLCESTSAID']?? ' ';
                }
                //item de entrada
                else{
                    $cdalmoxarife = $rowEntrada['CDALMOXENTR']?? ' ';
                    $cdlocalestoq = $rowEntrada['CDLCESTENTR']?? ' ';
                }

                $prod['NRLOTEESTQ']?? ' ';

                $prod['NRSUBLOTE']?? ' ';

                $prod['__is_new'] = false;

                $cdalmoxarifeCusto = $rowEntrada['CDALMOXSAID']?? ' ';
                $cdlocalestoqCusto = $rowEntrada['CDLCESTSAID']?? ' ';
                
                $vlrunitario  =  $prod['VRUNITLANC']?? $prod['VRUNITITPROCESSA'];

                $countproduto++;
                $nrsequitem   = str_pad($countproduto, 4, "0", STR_PAD_LEFT);
                var_dump($nrsequitem);die;
                $row = new Row();
                $nrsequitemrequest = null;
                if($saveNRSEQITEMREQUEST) {
                    if(isset($prod['NRSEQITEM'])) $nrsequitemrequest = $prod['NRSEQITEM'];
                }

                $row->set('__is_new',     true);
                $row->set('NRORG',        $nrorg);
                $row->set('NRORGULTATU',  $nrorg);
                $row->set('NRORGINCLUSAO', $nrorg);
                $row->set('DTULTATU',     $currentTime);
                $row->set('DTINCLUSAO',   $currentTime);
                $row->set('CDOPERINCLUSAO', $operador);
                $row->set('CDOPERULTATU', $operador);
                $row->set('IDATIVO',      'S');
                $row->set('CDOPERADOR',   $operador);
                $row->set('CDFILIAL',     $rowEntrada['CDFILIAL'] );
                $row->set('NRLANCESTQ',   $nrlancamentoestoque);
                $row->set('NRSEQUITEM',   $nrsequitem);
                $row->set('CDPRODUTO',    $prod['CDPRODUTO']);
                $row->set('CDALMOXARIFE', $cdalmoxarife);
                $row->set('CDLOCALESTOQ', $cdlocalestoq);
                $row->set('DSLANCESTIT',  'Transferência de Estoque '.$rowEntrada['NRCTRLMOVEST']);
                $row->set('NRLOTEESTQ',   $prod['NRLOTEESTQ']);
                $row->set('NRSUBLOTE',    $prod['NRSUBLOTE']);
                $row->set('CDFILIMOVI',   $rowEntrada['CDFILIMOVI']);
                if(!isset($prod['CDPRODESTO'])){
                    $consulta = $this->est57000transferenciaestoquefactory->getDataForProduct($prod['CDARVPROD']);
                    $prod['CDPRODESTO'] = $consulta['CDPRODESTO'];
                }
                $row->set('CDPRODMOVI',   $prod['CDPRODESTO']);
                $row->set('IDORDEMCLC',   $identradaSaida);
                $row->set('DTLANCMOVI' ,  $rowEntrada['DTMOVIESTQ']);
                $row->set('IDEVOLESTOQ',  'S');
                $row->set('IDTIPOMOVI',   $identradaSaida);
                $row->set('QTTOTLANCTO',  floatval ($prod['QTTOTLANCTO']));
                $row->set('QTLANCTOEST',  floatval ($prod['QTTOTLANCTO']) * ($prod['VRFATOCONV'] == 0 ? 1 : $prod['VRFATOCONV']));
                $row->set('VRUNILANCTO' , $vlrunitario);
                $row->set('VRTOTLANCTO' , $vlrunitario * floatval($prod['QTTOTLANCTO']));
                $row->set('VRESTOQDIA' ,  $vlrunitario * floatval($prod['QTTOTLANCTO']));
                $row->set('VRESTOQBRUT' , ($vlrunitario / ($prod['VRFATOCONV'] == 0 ? 1 : $prod['VRFATOCONV'])) * floatval($prod['QTTOTLANCTO']));
                $row->set('VRLANCTOEST' , $vlrunitario / ($prod['VRFATOCONV'] == 0 ? 1 : $prod['VRFATOCONV']));
                $row->set('VRLANCTOBRUT', $vlrunitario / ($prod['VRFATOCONV'] == 0 ? 1 : $prod['VRFATOCONV']));
                $row->set('NRLANCORIGRP',  null);
                $row->set('NRSEQITEMREQEST',  $nrsequitemrequest);
                $row->set('QTDEVOLESTOQ',  null);
                $arrayItens [] = $row;
            }
            $dataSet = new DataSet('itlanctoest_produto_itrequesto', $arrayItens);
            $this->dataSourceManager->persist($dataSet);
            $this->entityManager->flush();
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function buscaTipoCustoFilial($nrorg, $cdfilial) {
        $sql = "SELECT NVL(CDTIPCUSTEST,3) AS CDTIPCUSTEST
                FROM PARAMFILIAL
                WHERE
                    CDFILIAL = :P_CDFILIAL AND
                    NRORG    = :P_NRORG";
        $params = array(
           'P_NRORG'=>$nrorg,
           'P_CDFILIAL'=>$cdfilial
        );

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

}